==> netxms-1.1.2.tar.gz: netxms-1.1.2/src/libnxsl/array.cpp <==
/* 
** NetXMS - Network Management System
** NetXMS Scripting Language Interpreter
** Copyright (C) 2003-2010 Victor Kirhenshtein
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** File: array.cpp
**
**/

#include "libnxsl.h"


//
// Array constructors
//

NXSL_Array::NXSL_Array()
{
	m_refCount = 0;
	m_size = 0;
	m_allocated = 0;
	m_data = NULL;
}

NXSL_Array::NXSL_Array(NXSL_Array *src)
{
	int i;

	m_refCount = 0;
	m_size = src->m_size;
	m_allocated = src->m_size;
	if (m_size > 0)
	{
		m_data = (NXSL_ArrayElement *)malloc(sizeof(NXSL_ArrayElement) * m_size);
		for(i = 0; i < m_size; i++)
		{
			m_data[i].index = src->m_data[i].index;
			m_data[i].value = new NXSL_Value(src->m_data[i].value);
		}
	}
	else
	{
		m_data = NULL;
	}
}


//
// Array destructor
//

NXSL_Array::~NXSL_Array()
{
	int i;

	for(i = 0; i < m_size; i++)
		delete m_data[i].value;
	safe_free(m_data);
}


//
// Compare two ints
//

static int CompareElements(const void *p1, const void *p2)
{
	return COMPARE_NUMBERS(((NXSL_ArrayElement *)p1)->index, ((NXSL_ArrayElement *)p2)->index);
}


//
// Get element
//

NXSL_Value *NXSL_Array::get(int index)
{
	NXSL_ArrayElement *element, key;

	key.index = index;
	element = (NXSL_ArrayElement *)bsearch(&key, m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	return (element != NULL) ? element->value : NULL;
}


//
// Set element
//

void NXSL_Array::set(int index, NXSL_Value *value)
{
	NXSL_ArrayElement *element, key;

	key.index = index;
	element = (NXSL_ArrayElement *)bsearch(&key, m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	if (element != NULL)
	{
		delete element->value;
		element->value = value;
	}
	else
	{
		if (m_size == m_allocated)
		{
			m_allocated += 32;
			m_data = (NXSL_ArrayElement *)realloc(m_data, sizeof(NXSL_ArrayElement) * m_allocated);
		}
		m_data[m_size].index = index;
		m_data[m_size].value = value;
		m_size++;
		qsort(m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	}
}

==> netxms-2.0.6.tar.gz: netxms-2.0.6/src/libnxsl/array.cpp <==
/* 
** NetXMS - Network Management System
** NetXMS Scripting Language Interpreter
** Copyright (C) 2003-2015 Victor Kirhenshtein
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU Lesser General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** File: array.cpp
**
**/

#include "libnxsl.h"

/**
 * Create empty array
 */
NXSL_Array::NXSL_Array() : NXSL_HandleCountObject()
{
	m_size = 0;
	m_allocated = 0;
	m_data = NULL;
}

/**
 *  Create copy of given array
 */
NXSL_Array::NXSL_Array(const NXSL_Array *src) : NXSL_HandleCountObject()
{
	m_size = src->m_size;
	m_allocated = src->m_size;
	if (m_size > 0)
	{
		m_data = (NXSL_ArrayElement *)malloc(sizeof(NXSL_ArrayElement) * m_size);
		for(int i = 0; i < m_size; i++)
		{
			m_data[i].index = src->m_data[i].index;
			m_data[i].value = new NXSL_Value(src->m_data[i].value);
		}
	}
	else
	{
		m_data = NULL;
	}
}

/**
 * Create array populated with values from string list
 */
NXSL_Array::NXSL_Array(const StringList *values) : NXSL_HandleCountObject()
{
	m_size = values->size();
	m_allocated = m_size;
	if (m_size > 0)
	{
		m_data = (NXSL_ArrayElement *)malloc(sizeof(NXSL_ArrayElement) * m_size);
		for(int i = 0; i < m_size; i++)
		{
			m_data[i].index = i;
			m_data[i].value = new NXSL_Value(values->get(i));
		}
	}
	else
	{
		m_data = NULL;
	}
}

/**
 * Destructor
 */
NXSL_Array::~NXSL_Array()
{
	int i;

	for(i = 0; i < m_size; i++)
		delete m_data[i].value;
	safe_free(m_data);
}

/**
 * Compare two indexes
 */
static int CompareElements(const void *p1, const void *p2)
{
	return COMPARE_NUMBERS(((NXSL_ArrayElement *)p1)->index, ((NXSL_ArrayElement *)p2)->index);
}

/**
 * Get element by index
 */
NXSL_Value *NXSL_Array::get(int index) const
{
	NXSL_ArrayElement *element, key;

	key.index = index;
	element = (NXSL_ArrayElement *)bsearch(&key, m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	return (element != NULL) ? element->value : NULL;
}

/**
 * Get element by internal position (used by iterator)
 */
NXSL_Value *NXSL_Array::getByPosition(int position) const
{
	if ((position < 0) || (position >= m_size))
		return NULL;
	return m_data[position].value;
}

/**
 * Get all elements as string list
 */
StringList *NXSL_Array::toStringList() const
{
   StringList *list = new StringList();
   for(int i = 0; i < m_size; i++)
   {
      list->add(m_data[i].value->getValueAsCString());
   }
   return list;
}

/**
 * Set element
 */
void NXSL_Array::set(int index, NXSL_Value *value)
{
	NXSL_ArrayElement *element, key;

	key.index = index;
	element = (NXSL_ArrayElement *)bsearch(&key, m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	if (element != NULL)
	{
		delete element->value;
		element->value = value;
	}
	else
	{
		if (m_size == m_allocated)
		{
			m_allocated += 32;
			m_data = (NXSL_ArrayElement *)realloc(m_data, sizeof(NXSL_ArrayElement) * m_allocated);
		}
		m_data[m_size].index = index;
		m_data[m_size].value = value;
		m_size++;
		qsort(m_data, m_size, sizeof(NXSL_ArrayElement), CompareElements);
	}
}
